@@ -1,6 +1,10 @@
+from dataclasses import dataclass
+from typing import List
+import inspect
+
 import requests
 
-from .response import Response
+from flask_discord_interactions.response import Response
 
 
 class CommandOptionType:
@@ -26,7 +30,26 @@ class ChannelType:
     GUILD_STORE = 6
 
 
-class User:
+class ContextObject:
+    @classmethod
+    def from_dict(cls, data):
+        """
+        Construct the Context object from a dictionary, skipping any keys
+        in the dictionary that do not correspond to fields of the class.
+
+        Parameters
+        ----------
+        data
+            A dictionary of fields to set on the Context object.
+        """
+        return cls(**{
+            k: v for k, v in data.items()
+            if k in inspect.signature(cls).parameters
+        })
+
+
+@dataclass
+class User(ContextObject):
     """
     Represents a User (the identity of a Discord user, not tied to any
     specific guild).
@@ -56,20 +79,22 @@ class User:
     public_flags
         Miscellaneous information about the user.
     """
+    id: str = None
+    username: str = None
+    discriminator: str = None
+    avatar_hash: str = None
+    bot: bool = None
+    system: bool = None
+    mfa_enabled: bool = None
+    locale: str = None
+    flags: int = None
+    premium_type: int = None
+    public_flags: int = None
 
-    def __init__(self, data=None):
-        if data:
-            self.id = data.get("id")
-            self.username = data.get("username")
-            self.discriminator = data.get("discriminator")
-            self.avatar_hash = data.get("avatar")
-            self.bot = data.get("bot", False)
-            self.system = data.get("system", False)
-            self.mfa_enabled = data.get("mfa_enabled", False)
-            self.locale = data.get("locale")
-            self.flags = data.get("flags")
-            self.premium_type = data.get("premium_type")
-            self.public_flags = data.get("public_flags")
+    @classmethod
+    def from_dict(cls, data):
+        data = {**data, **data.get("user", {})}
+        return super().from_dict(data)
 
     @property
     def display_name(self):
@@ -83,6 +108,7 @@ class User:
                 f"{self.id}/{self.avatar_hash}.png")
 
 
+@dataclass
 class Member(User):
     """
     Represents a Member (a specific Discord :class:`User` in one particular
@@ -105,18 +131,13 @@ class Member(User):
     pending
         Whether the user has passed the membership requirements of a guild.
     """
-
-    def __init__(self, data=None):
-        if data:
-            super().__init__(data["user"])
-
-            self.nick = data.get("nick")
-            self.roles = data.get("roles")
-            self.joined_at = data.get("joined_at")
-            self.premium_since = data.get("premium_since")
-            self.deaf = data.get("deaf")
-            self.mute = data.get("mute")
-            self.pending = data.get("pending")
+    nick: str = None
+    roles: list = None
+    joined_at: str = None
+    premium_since: str = None
+    deaf: bool = None
+    mute: bool = None
+    pending: bool = None
 
     @property
     def display_name(self):
@@ -127,7 +148,8 @@ class Member(User):
         return self.nick or self.username
 
 
-class Channel:
+@dataclass
+class Channel(ContextObject):
     """
     Represents a Channel in Discord. This includes voice channels, text
     channels, and channel categories.
@@ -143,16 +165,14 @@ class Channel:
     type
         The type of channel.
     """
-
-    def __init__(self, data=None):
-        if data:
-            self.id = data.get("id")
-            self.name = data.get("name")
-            self.permissions = data.get("permissions")
-            self.type = data.get("type")
+    id: str = None
+    name: str = None
+    permissions: int = None
+    type: int = None
 
 
-class Role:
+@dataclass
+class Role(ContextObject):
     """
     Represents a Role in Discord.
 
@@ -177,21 +197,18 @@ class Role:
     tags
         Miscellaneous information about the role.
     """
-
-    def __init__(self, data=None):
-        if data:
-            self.id = data.get("id")
-            self.name = data.get("name")
-            self.color = data.get("color")
-            self.hoist = data.get("hoist")
-            self.position = data.get("position")
-            self.permissions = data.get("permissions")
-            self.managed = data.get("managed")
-            self.mentionable = data.get("mentionable")
-            self.tags = data.get("tags", {})
+    id: str = None
+    name: str = None
+    color: str = None
+    hoist: bool = None
+    position: int = None
+    managed: bool = None
+    mentionable: bool = None
+    tags: dict = None
 
 
-class Context:
+@dataclass
+class Context(ContextObject):
     """
     Represents the context in which a :class:`SlashCommand` is invoked.
 
@@ -220,22 +237,40 @@ class Context:
     roles
         :class:`Role` object for each role specified as an option.
     """
+    author: Member = None
+    id: str = None
+    token: str = None
+    channel_id: str = None
+    guild_id: str = None
+    options: list = None
+    command_name: str = None
+    command_id: str = None
+    members: List[Member] = None
+    channels: List[Channel] = None
+    roles: List[Role] = None
 
-    def __init__(self, discord, app, data=None):
-        self.client_id = app.config["DISCORD_CLIENT_ID"]
-        self.auth_headers = discord.auth_headers(app)
 
-        if data:
-            self.author = Member(data["member"])
-            self.id = data["id"]
-            self.token = data["token"]
-            self.channel_id = data["channel_id"]
-            self.guild_id = data["guild_id"]
-            self.options = data["data"].get("options")
-            self.command_name = data["data"]["name"]
-            self.command_id = data["data"]["id"]
+    @classmethod
+    def from_data(cls, discord=None, app=None, data={}):
+        if data is None:
+            data = {}
 
-            self.parse_resolved(data["data"].get("resolved", {}))
+        self = cls()
+
+        self.client_id = app.config["DISCORD_CLIENT_ID"] if app else ""
+        self.auth_headers = discord.auth_headers(app) if discord else {}
+
+        self.author = Member.from_dict(data.get("member", {}))
+        self.id = data.get("id")
+        self.token = data.get("token")
+        self.channel_id = data.get("channel_id")
+        self.guild_id = data.get("guild_id")
+        self.options = data.get("data", {}).get("options")
+        self.command_name = data.get("data", {}).get("name")
+        self.command_id = data.get("data", {}).get("id")
+
+        self.parse_resolved(data.get("data", {}).get("resolved", {}))
+        return self
 
     def parse_resolved(self, data):
         """
@@ -254,12 +289,12 @@ class Context:
         for id in data.get("members", {}):
             member_info = data["members"][id]
             member_info["user"] = data["users"][id]
-            self.members[id] = Member(member_info)
+            self.members[id] = Member.from_dict(member_info)
 
-        self.channels = {id: Channel(data)
+        self.channels = {id: Channel.from_dict(data)
                          for id, data in data.get("channels", {}).items()}
 
-        self.roles = {id: Role(data)
+        self.roles = {id: Role.from_dict(data)
                       for id, data in data.get("roles", {}).items()}
 
     def create_args(self, data, resolved):
@@ -296,12 +331,12 @@ class Context:
                 member_data = resolved["members"][option["value"]]
                 member_data["user"] = resolved["users"][option["value"]]
 
-                kwargs[option["name"]] = Member(member_data)
+                kwargs[option["name"]] = Member.from_dict(member_data)
             elif option["type"] == CommandOptionType.CHANNEL:
-                kwargs[option["name"]] = Channel(
+                kwargs[option["name"]] = Channel.from_dict(
                     resolved["channels"][option["value"]])
             elif option["type"] == CommandOptionType.ROLE:
-                kwargs[option["name"]] = Role(
+                kwargs[option["name"]] = Role.from_dict(
                     resolved["roles"][option["value"]])
             else:
                 kwargs[option["name"]] = option["value"]
